@@ -1,10 +1,10 @@
-# Copyright (c) 1998-2003 Graham Barr <gbarr@pobox.com>. All rights reserved.
+# Copyright (c) 1998-2008 Graham Barr <gbarr@pobox.com>. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 
 package Net::LDAP::Constant;
 
-$VERSION = "0.02";
+$VERSION = "0.05";
 
 use Carp;
 
@@ -135,32 +135,60 @@ error codes.
 
 Operation completed without error
 
+=item LDAP_PP_PASSWORD_EXPIRED (0)
+
+The account's password has expired.
+
 =item LDAP_OPERATIONS_ERROR (1)
 
 Server encountered an internal error
 
+=item LDAP_PP_ACCOUNT_LOCKED (1)
+
+The account is locked.
+
 =item LDAP_PROTOCOL_ERROR (2)
 
 Unrecognized version number or incorrect PDU structure
 
+=item LDAP_PP_CHANGE_AFTER_RESET (2)
+
+The account's password has been reset and now must be changed.
+
 =item LDAP_TIMELIMIT_EXCEEDED (3)
 
 The time limit on a search operation has been exceeded
 
+=item LDAP_PP_PASSWORD_MOD_NOT_ALLOWED (3)
+
+The account's password may not be modified.
+
 =item LDAP_SIZELIMIT_EXCEEDED (4)
 
 The maximum number of search results to return has been exceeded.
 
+=item LDAP_PP_MUST_SUPPLY_OLD_PASSWORD (4)
+
+The old password must also be supplied when setting a new password.
+
 =item LDAP_COMPARE_FALSE (5)
 
 This code is returned when a compare request completes and the attribute value
 given is not in the entry specified
 
+=item LDAP_PP_INSUFFICIENT_PASSWORD_QUALITY (5)
+
+The new password was not of sufficient quality.
+
 =item LDAP_COMPARE_TRUE (6)
 
 This code is returned when a compare request completes and the attribute value
 given is in the entry specified
 
+=item LDAP_PP_PASSWORD_TOO_SHORT (6)
+
+The new password was too short.
+
 =item LDAP_AUTH_METHOD_NOT_SUPPORTED (7)
 
 Unrecognized SASL mechanism name
@@ -169,10 +197,18 @@ Unrecognized SASL mechanism name
 
 Unrecognized SASL mechanism name
 
+=item LDAP_PP_PASSWORD_TOO_YOUNG (7)
+
+The previous password was changed too recently.
+
 =item LDAP_STRONG_AUTH_REQUIRED (8)
 
 The server requires authentication be performed with a SASL mechanism
 
+=item LDAP_PP_PASSWORD_IN_HISTORY (8)
+
+The new password was used too recently.
+
 =item LDAP_PARTIAL_RESULTS (9)
 
 Returned to version 2 clients when a referral is returned. The response
@@ -441,6 +477,8 @@ The referral hop limit has been exceeded.
 
 =head2 Control OIDs
 
+=over 4
+
 =item LDAP_CONTROL_SORTREQUEST (1.2.840.113556.1.4.473)
 
 =item LDAP_CONTROL_SORTRESULT (1.2.840.113556.1.4.474)
@@ -449,7 +487,7 @@ The referral hop limit has been exceeded.
 
 =item LDAP_CONTROL_VLVRESPONSE (2.16.840.1.113730.3.4.10)
 
-=item LDAP_CONTROL_PROXYAUTHENTICATION (2.16.840.1.113730.3.4.12)
+=item LDAP_CONTROL_PROXYAUTHENTICATION (2.16.840.1.113730.3.4.18)
 
 =item LDAP_CONTROL_PAGED (1.2.840.113556.1.4.319)
 
@@ -457,6 +495,8 @@ The referral hop limit has been exceeded.
 
 =item LDAP_CONTROL_MATCHEDVALS (1.2.826.0.1.3344810.2.2)
 
+=item LDAP_CONTROL_MATCHEDVALUES (1.2.826.0.1.3344810.2.3)
+
 =item LDAP_CONTROL_MANAGEDSAIT (2.16.840.1.113730.3.4.2)
 
 =item LDAP_CONTROL_PERSISTENTSEARCH (2.16.840.1.113730.3.4.3)
@@ -469,6 +509,10 @@ The referral hop limit has been exceeded.
 
 =item LDAP_CONTROL_REFERRALS (1.2.840.113556.1.4.616)
 
+=item LDAP_CONTROL_PASSWORDPOLICY (1.3.6.1.4.1.42.2.27.8.5.1)
+
+=back
+
 =head2 Extension OIDs
 
 B<Net::LDAP::Constant> exports constant subroutines for the following LDAP
@@ -478,7 +522,15 @@ extension OIDs.
 
 =item LDAP_EXTENSION_START_TLS (1.3.6.1.4.1.1466.20037)
 
-Indicates if the server supports the Start TLS extension (RFC-2830)
+Indicates if the server supports the Start TLS extension (RFC 2830)
+
+=item LDAP_EXTENSION_PASSWORD_MODIFY (1.3.6.1.4.1.4203.1.11.1)
+
+Indicates that the server supports the Password Modify extension (RFC 3062)
+
+=item LDAP_EXTENSION_WHO_AM_I (1.3.6.1.4.1.4203.1.11.3)
+
+Indicates that the server supports the "Who am I?" extension (draft-zeilenga-ldap-authzid-09)
 
 =back
 
@@ -496,12 +548,8 @@ E<lt>perl-ldap@perl.orgE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1998-2003 Graham Barr. All rights reserved. This program is
+Copyright (c) 1998-2008 Graham Barr. All rights reserved. This program is
 free software; you can redistribute it and/or modify it under the same
 terms as Perl itself.
 
-=for html <hr>
-
-I<$Id: Constant.pm,v 1.9 2003/06/02 15:13:10 gbarr Exp $>
-
 =cut
